import os
import shutil
import re
import copy

import yaml
from petroglyph.post import Post
from petroglyph import logger


def process_template(template, args):
    placeholders = set(re.findall('{{([a-zA-Z][a-zA-Z0-9_-]*)}}', template))
    for placeholder in placeholders:
        if placeholder in args:
            template = template.replace('{{' + placeholder + '}}', str(args[placeholder]))
    return template


def generate(regenerate=False, dry_run=False):
    cwd = os.getcwd()
    with open(os.path.join(cwd, 'config.yaml'), 'rb') as f:
        config = yaml.safe_load(f)
    drafts = [file for file in os.listdir(os.path.join(cwd, 'posts')) if file[0] != '.']
    posts = []
    for draft in drafts:
        posts.append(Post(file=os.path.join('posts', draft)))
    skin = {}
    with open(os.path.join('skin', 'post.html'), 'rb') as f:
        skin['post'] = f.read()
    with open(os.path.join('skin', 'post-peek.html'), 'rb') as f:
        skin['post-peek'] = f.read()
    with open(os.path.join('skin', 'home.html')) as f:
        skin['home'] = f.read()
    with open(os.path.join('skin', 'tag.html')) as f:
        skin['tag'] = f.read()
    if not os.path.isdir('blog'):
        os.mkdir('blog')
    if os.path.exists('.petroglyph-metadata'):
        with open('.petroglyph-metadata', 'rb') as f:
            posts_meta = yaml.safe_load(f)
    else:
        posts_meta = {}
    post_previews_text = []
    os.chdir('blog')
    logger.log("Found %d post%s." % (len(posts), '' if len(posts) == 1 else 's'))
    stats = {}
    stats['new_posts'] = 0
    stats['changed_posts'] = 0
    stats['regenerated_posts'] = 0
    stats['generated_posts'] = 0
    current_posts_slugs = []

    for post in posts:
        current_posts_slugs.append(post.slug)
        if post.slug in posts_meta:
            post.mtime = posts_meta[post.slug]['mtime']
            new = False
        else:
            new = True
            stats['new_posts'] += 1
        if not os.path.exists(post.slug):
            os.mkdir(post.slug)
        post_hash = post.get_hash()
        if not new and post_hash != posts_meta[post.slug]['hash']:
            stats['changed_posts'] += 1
        if regenerate or new or post_hash != posts_meta[post.slug]['hash']:
            posts_meta[post.slug] = {'mtime': post.mtime, 'hash': post_hash}
            post_args = {
                'title': post.title,
                'tags': ''.join(
                    ['<a href="../tag/' + tag + '" class="tag">#' + tag + '</a>'
                        for tag in post.tags]
                ),
                'blog_title': config['title'],
                'date': post.getmtime(),
                'content': post.get_html()
            }
            post_data = copy.deepcopy(post.front_matter_data)
            post_data.update(post_args)
            if not dry_run:
                if not new:
                    stats['regenerated_posts'] += 1
                else:
                    stats['generated_posts'] += 1
                with open(os.path.join(post.slug, 'index.html'), 'wb') as post_file:
                    post_file.write(process_template(skin['post'], post_data))

    posts.sort(key=lambda p: p.get_time(), reverse=True)
    tag_data = {}
    for post in posts:
        post_peek_args = {
            'slug': post.slug,
            'title': post.title,
            'date': post.getmtime(),
            'peek': post.get_preview(),
            'tags': ''.join(
                ['<a href="tag/' + tag + '" class="tag">#' + tag + '</a>'
                    for tag in post.tags]
            )
        }
        post_peek_data = copy.deepcopy(post.front_matter_data)
        post_peek_data.update(post_peek_args)
        post_previews_text.append(process_template(skin['post-peek'], post_peek_data))
        post_peek_data.update({
            'tags': ''.join(
                ['<a href="../../tag/' + tag + '" class="tag">#' + tag + '</a>'
                    for tag in post.tags]
            ),
            'slug': '../../' + post.slug
        })
        for tag in post.tags:
            if tag not in tag_data:
                tag_data[tag] = []
            tag_data[tag].append(process_template(skin['post-peek'], post_peek_data))
    home_args = {
        'title': config['title'],
        'author': config['author'],
        'description': config['description'],
        'posts': "\n".join(post_previews_text)
    }
    tag_args = {
        'title': config['title'],
        'author': config['author'],
        'description': config['description'],
    }
    if stats['new_posts']:
        logger.log(
            "%s new post%s." % (
                stats['new_posts'],
                '' if stats['new_posts'] == 1 else 's'
            ),
            logger.SUCCESS
        )
    if stats['changed_posts']:
        logger.log(
            "%s changed post%s." % (
                stats['changed_posts'],
                '' if stats['changed_posts'] == 1 else 's'
            ),
            logger.WARNING
        )
    if not dry_run and (regenerate or stats['new_posts'] + stats['changed_posts'] > 0):
        with open('index.html', 'wb') as f:
            f.write(process_template(skin['home'], home_args))
        if not os.path.isdir('tag'):
            os.mkdir('tag')
        for tag in tag_data:
            if not os.path.isdir(os.path.join('tag', tag)):
                os.mkdir(os.path.join('tag', tag))
            tag_args.update(
                {
                    'tag': '#' + tag,
                    'posts': "\n".join(tag_data[tag])
                }
            )
            with open(os.path.join('tag', tag, 'index.html'), 'wb') as f:
                f.write(process_template(skin['tag'], tag_args))

    deleted_posts = set(posts_meta).difference(current_posts_slugs)
    stats['deleted_posts'] = len(deleted_posts)

    if stats['deleted_posts']:
        logger.log(
            "%s deleted post%s." % (
                stats['deleted_posts'],
                '' if stats['deleted_posts'] == 1 else 's'
            ),
            logger.ERROR
        )

    if stats['new_posts'] + stats['changed_posts'] + stats['deleted_posts'] == 0:
        logger.log("No changes to make.")

    if not dry_run:
        for deleted_post in deleted_posts:
            del posts_meta[deleted_post]
            try:
                shutil.rmtree(deleted_post)
            except:
                logger.log("Failed to delete '%s'." % deleted_post, logger.ERROR)
        if stats['deleted_posts']:
            logger.log(
                "Deleted %s old post%s." % (
                    stats['deleted_posts'],
                    '' if stats['deleted_posts'] == 1 else 's'
                ),
                logger.SUCCESS
            )
    os.chdir(os.pardir)
    if not dry_run:
        with open('.petroglyph-metadata', 'wb') as f:
            yaml.dump(posts_meta, f)
        paths = os.listdir('skin')
        for path in paths:
            if os.path.isdir(os.path.join('skin', path)):
                if os.path.exists(os.path.join('blog', path)):
                    shutil.rmtree(os.path.join('blog', path))
                shutil.copytree(os.path.join('skin', path), os.path.join('blog', path))
    if ((stats['generated_posts'] or stats['regenerated_posts'] or stats['deleted_posts'])
            and not dry_run):
        if stats['generated_posts']:
            logger.log(
                "Generated %s new post%s." % (
                    stats['generated_posts'],
                    '' if stats['generated_posts'] == 1 else 's'
                ),
                logger.SUCCESS
            )
        if stats['regenerated_posts']:
            logger.log(
                "Regenerated %s post%s." % (
                    stats['regenerated_posts'],
                    '' if stats['regenerated_posts'] == 1 else 's'
                ),
                logger.SUCCESS
            )
    if not dry_run:
        logger.log("Done.", logger.SUCCESS)



import shutil
import os
from petroglyph import logger


def generate_config():
    logger.log("Configuring settings...")
    if os.path.exists('config.yaml'):
        logger.log(
            "The file "
            + os.path.join(os.getcwd(), "config.yaml")
            + " already exists.",
            logger.WARNING
        )
        print "Would you like to replace it? (y/n)",
        response = raw_input()
        if response.lower() != 'y':
            return
    blog_config = {}
    blog_title = raw_input("Blog title: ").strip()
    if blog_title is not '':
        blog_config['title'] = blog_title
    blog_author = raw_input("Blog author: ").strip()
    if blog_author is not '':
        blog_config['author'] = blog_author
    blog_description = raw_input("Blog description: ").strip()
    if blog_description is not '':
        blog_config['description'] = blog_description
    import yaml
    f = open('config.yaml', 'w')
    yaml.dump(blog_config, f, default_flow_style=False)
    logger.log("Saved configuration in config.yaml.", logger.SUCCESS)


def init(skin):
    skin = str(skin).strip()
    petroglyph_root = os.path.dirname(os.path.abspath(__file__))
    logger.log("Copying skin '%s'..." % skin)
    if not os.path.exists(os.path.join(petroglyph_root, 'skins', skin)):
        logger.log("Skin '%s' does not exist. Exiting." % skin, logger.ERROR)
        import sys
        sys.exit(1)
    try:
        shutil.copytree(os.path.join(petroglyph_root, 'skins', skin), 'skin')
    except OSError:
        logger.log(
            "The path "
            + os.path.join(os.getcwd(), 'skin')
            + " already exists.",
            logger.WARNING
        )
        print "Would you like to replace it? (y/n)",
        response = raw_input()
        if response.lower() == 'y':
            logger.log("Replacing skin...")
            try:
                shutil.rmtree('skin')
                shutil.copytree(os.path.join(petroglyph_root, 'skins', skin), 'skin')
            except OSError:
                logger.log("Error: Could not replace skin. Exiting.", logger.ERROR)
                exit()
    logger.log("Creating posts directory...")
    try:
        os.mkdir('posts')
    except OSError:
        logger.log("Directory exists.", logger.SUCCESS)
    generate_config()
    logger.log("Petroglyph initialized.", logger.SUCCESS)



from setuptools import setup

setup(
    name='petroglyph',
    version='0.4.0',
    description='A static blog generator',
    url='https://github.com/polybuildr/petroglyph',
    author='Vivek Ghaisas',
    license='MIT',
    keywords='static blog',
    packages=['petroglyph'],
    install_requires=['mistune', 'pyyaml', 'docutils'],
    include_package_data=True,
    scripts=['petroglyph/petroglyph']
)



import os
from datetime import datetime, time
import hashlib


class Post:
    def __init__(self, file):
        if file[-4:] == '.rst':
            self.text_type = 'rst'
        else:
            self.text_type = 'md'
        f = open(file, 'r')
        text = f.read()
        import yaml
        import re
        match = re.search(r'^\s*---(.*?)---\s*(.*)$', text, re.DOTALL | re.MULTILINE)
        if match is None:
            raise ValueError("YAML front-matter missing from '%s'." % file)
        self.front_matter = match.group(1)
        config = yaml.safe_load(self.front_matter)
        if config is None:
            raise ValueError("Empty YAML front-matter in '%s'." % file)
        if 'title' not in config:
            raise ValueError("'%s' does not have title in YAML front-matter." % file)
        self.front_matter_data = config
        self.title = config['title']
        self.text = match.group(2)
        self.overrides = []
        if 'date' in config:
            self.overrides.append('date')
            self.custom_date = datetime.combine(config['date'], time.min)

        self.mtime = datetime.fromtimestamp(os.path.getmtime(file))
        if 'tags' in config:
            self.tags = [tag.strip() for tag in config['tags'].split(',')]
        else:
            self.tags = []
        file = os.path.basename(file)
        if file[-3:] == '.md':
            self.slug = file[:-3]
        else:
            self.slug = file
        if self.slug in ('js', 'css', 'search', 'pages', 'special'):
            raise ValueError("Title '%s' is a reserved title ('%s')." % (self.slug, file))

    def __str__(self):
        return self.slug

    def __repr__(self):
        return str(self)

    def get_html(self):
        if self.text_type == 'rst':
            from docutils.core import publish_parts
            return publish_parts(self.text, writer_name='html')['html_body']
        else:
            import mistune
            return mistune.markdown(self.text)

    def get_preview(self):
        import re
        match = re.search(r'(.*)<!--\s*more\s*-->', self.get_html(), re.MULTILINE | re.DOTALL)
        if not match:
            return self.get_html()
        return match.group(1)

    def getmtime(self):
        def suffix(d):
            return 'th' if 11 <= d <= 13 else {1: 'st', 2: 'nd', 3: 'rd'}.get(d % 10, 'th')
        if 'date' in self.overrides:
            mtime = self.custom_date
        else:
            mtime = self.mtime
        return mtime.strftime('{S} %B, %Y').replace(
            '{S}',
            str(mtime.day) + suffix(mtime.day)
        )

    def get_time(self):
        if 'date' in self.overrides:
            return self.custom_date
        return self.mtime

    def get_hash(self):
        return hashlib.md5(self.front_matter + self.text).hexdigest()



class tcolors:
    BLUE = '\033[38;5;4m'
    GREEN = '\033[38;5;2m'
    ORANGE = '\033[38;5;3m'
    RED = '\033[38;5;1m'
    END = '\033[0m'

INFO, SUCCESS, WARNING, ERROR = range(4)


def log(message, type=INFO):
    if type is INFO:
        print tcolors.BLUE + message + tcolors.END
    elif type is SUCCESS:
        print tcolors.GREEN + message + tcolors.END
    elif type is WARNING:
        print tcolors.ORANGE + message + tcolors.END
    elif type is ERROR:
        print tcolors.RED + message + tcolors.END
    else:
        print message
